import { ReduxActionContextDecorator, ReduxActionDecorator, ReduxActionDispatcher, ReduxReducerDecorator, ReduxStateDecorator } from '@harmowatch/redux-decorators';
import 'rxjs/add/operator/toPromise';
import { AsyncSubject as AsyncSubject$1 } from 'rxjs/AsyncSubject';
import { Observable as Observable$1 } from 'rxjs/Observable';
import { Inject, Injectable, InjectionToken, Injector, NgModule, Optional, Pipe, isDevMode } from '@angular/core';
import 'rxjs/add/operator/distinctUntilChanged';
import 'rxjs/add/operator/map';
import { CommonModule } from '@angular/common';
import 'rxjs/add/operator/takeWhile';
import { BehaviorSubject as BehaviorSubject$1 } from 'rxjs/BehaviorSubject';
import { createStore } from 'redux';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class ActionInterface {
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
const ReduxAction = ReduxActionDecorator.forMethod;
const ReduxActionContext = ReduxActionContextDecorator.forClass;

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @record
 */

class Registry {
    /**
     * @return {?}
     */
    static reset() {
        Registry._store = new AsyncSubject$1();
        Registry._reducers = [];
    }
    /**
     * @param {?} store
     * @return {?}
     */
    static registerStore(store) {
        Registry._store.next(store);
        Registry._store.complete();
        ReduxActionDispatcher.dispatchedActions.subscribe(action => {
            store.dispatch(/** @type {?} */ (action));
        });
    }
    /**
     * @param {?} stateName
     * @param {?} types
     * @param {?} reducer
     * @return {?}
     */
    static registerReducer(stateName, types, reducer) {
        if (Array.isArray(types)) {
            types.forEach((type) => {
                Registry._reducers.push({
                    reducer,
                    stateName,
                    type,
                });
            });
        }
        else {
            Registry._reducers.push({
                reducer,
                stateName,
                type: /** @type {?} */ (types),
            });
        }
    }
    /**
     * @param {?} state
     * @return {?}
     */
    static registerState(state) {
        Registry.getStore().then((store) => {
            const /** @type {?} */ stateConfig = ReduxStateDecorator.get(state.constructor);
            const /** @type {?} */ initState = state.getInitialState();
            let /** @type {?} */ initStateToResolve = initState;
            if (initState instanceof Observable$1) {
                initStateToResolve = initState.toPromise();
            }
            Promise.resolve(initStateToResolve).then((initialState) => {
                store.dispatch({
                    payload: {
                        initialValue: initialState,
                        name: stateConfig.name,
                    },
                    type: Registry.ACTION_REGISTER_STATE,
                });
            });
        });
    }
    /**
     * @param {?} type
     * @return {?}
     */
    static getReducerItemsByType(type) {
        const /** @type {?} */ reducerItemsByType = Registry._reducers
            .filter(reducerItem => ReduxActionDispatcher.getType(reducerItem.type) === type);
        return reducerItemsByType;
    }
    /**
     * @return {?}
     */
    static getStore() {
        return new Promise(Registry._store.subscribe.bind(Registry._store));
    }
}
Registry.ACTION_REGISTER_STATE = `@harmowatch/ngx-redux-core/registerState`;
Registry._store = new AsyncSubject$1();
Registry._reducers = [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class ReduxModuleRootReducer {
    /**
     * @param {?} state
     * @param {?} action
     * @return {?}
     */
    static reduce(state, action) {
        if (action.type === Registry.ACTION_REGISTER_STATE) {
            return ReduxModuleRootReducer.reduceRegisterState(state, /** @type {?} */ (action));
        }
        return ReduxModuleRootReducer.reduceByRegisteredReducers(state, action);
    }
    /**
     * @param {?} state
     * @param {?} action
     * @return {?}
     */
    static reduceRegisterState(state, action) {
        const { name, initialValue } = action.payload;
        return Object.assign({}, state, {
            [name]: initialValue,
        });
    }
    /**
     * @param {?} state
     * @param {?} action
     * @return {?}
     */
    static reduceByRegisteredReducers(state, action) {
        const /** @type {?} */ reducers = Registry.getReducerItemsByType(action.type);
        return reducers.reduce((stateToReduce, reducerItem) => {
            const { stateName, reducer } = reducerItem;
            return Object.assign({}, stateToReduce, {
                [stateName]: reducer(stateToReduce[stateName], action),
            });
        }, state);
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
const ReduxReducer = ReduxReducerDecorator.forMethod;

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class ReduxSelector extends Observable$1 {
    /**
     * @param {?=} selector
     * @param {?=} stateProvider
     */
    constructor(selector = '/', stateProvider) {
        super(observer => {
            Registry.getStore().then(store => {
                const /** @type {?} */ next = () => observer.next(ReduxSelector.getValueByState(store.getState(), selector, stateProvider));
                store.subscribe(() => next());
                next(); // we need to trigger a initial value, otherwise we've to wait until the first state change
            });
        });
        if (!ReduxSelectorCacheFactory.has(selector, stateProvider)) {
            ReduxSelectorCacheFactory.set(this, selector, stateProvider);
        }
    }
    /**
     * @template T
     * @param {?} selector
     * @param {?=} stateProvider
     * @return {?}
     */
    static create(selector, stateProvider) {
        return new ReduxSelector(selector, stateProvider);
    }
    /**
     * @param {?} selector
     * @param {?=} stateProvider
     * @return {?}
     */
    static getAbsoluteSelector(selector, stateProvider) {
        if (!selector.startsWith(ReduxSelector.DELIMITER)) {
            if (!stateProvider) {
                throw new Error('You need to provide a state provider, if you use relative selectors');
            }
            return `/${ReduxStateDecorator.get(stateProvider).name}/${selector}`;
        }
        return selector;
    }
    /**
     * @template S
     * @param {?} state
     * @param {?} selector
     * @param {?=} stateProvider
     * @return {?}
     */
    static getValueByState(state, selector, stateProvider) {
        const /** @type {?} */ value = ReduxSelector.getAbsoluteSelector(selector, stateProvider).split(ReduxSelector.DELIMITER)
            .filter(propertyKey => propertyKey !== '')
            .reduce((previousValue, propertyKey) => {
            if (!previousValue || !previousValue.hasOwnProperty(propertyKey)) {
                return null;
            }
            return previousValue[propertyKey];
        }, /** @type {?} */ (state));
        return value;
    }
}
ReduxSelector.DELIMITER = '/';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class ReduxSelectorCacheFactory {
    /**
     * @return {?}
     */
    static clear() {
        ReduxSelectorCacheFactory.selectors = {};
    }
    /**
     * @template T
     * @param {?} selector
     * @param {?=} stateProvider
     * @return {?}
     */
    static getOrCreate(selector, stateProvider) {
        if (ReduxSelectorCacheFactory.has(selector, stateProvider)) {
            return ReduxSelectorCacheFactory.get(selector, stateProvider);
        }
        return new ReduxSelector(selector, stateProvider);
    }
    /**
     * @template T
     * @param {?} selector
     * @param {?=} stateProvider
     * @return {?}
     */
    static get(selector, stateProvider) {
        const /** @type {?} */ key = ReduxSelectorCacheFactory.getKey(selector, stateProvider);
        return /** @type {?} */ (ReduxSelectorCacheFactory.selectors[key]);
    }
    /**
     * @param {?} reduxSelector
     * @param {?} selector
     * @param {?=} stateProvider
     * @return {?}
     */
    static set(reduxSelector, selector, stateProvider) {
        const /** @type {?} */ key = ReduxSelectorCacheFactory.getKey(selector, stateProvider);
        ReduxSelectorCacheFactory.selectors[key] = reduxSelector;
    }
    /**
     * @param {?} selector
     * @param {?=} stateProvider
     * @return {?}
     */
    static has(selector, stateProvider) {
        const /** @type {?} */ key = ReduxSelectorCacheFactory.getKey(selector, stateProvider);
        return !!ReduxSelectorCacheFactory.selectors[key];
    }
    /**
     * @param {?} selector
     * @param {?=} stateProvider
     * @return {?}
     */
    static getKey(selector, stateProvider) {
        return ReduxSelector.getAbsoluteSelector(selector, stateProvider);
    }
}
ReduxSelectorCacheFactory.selectors = {};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class StateDefToken extends InjectionToken {
    constructor() {
        super('StateDefToken');
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class ReduxSelectPipe {
    /**
     * @param {?=} stateDefs
     */
    constructor(stateDefs = []) {
        this.stateDef = stateDefs[0];
    }
    /**
     * @param {?} selector
     * @return {?}
     */
    transform(selector) {
        return ReduxSelectorCacheFactory.getOrCreate(selector, this.stateDef.provider).distinctUntilChanged();
    }
}
ReduxSelectPipe.decorators = [
    { type: Pipe, args: [{ name: 'reduxSelect' },] },
];
/** @nocollapse */
ReduxSelectPipe.ctorParameters = () => [
    { type: Array, decorators: [{ type: Inject, args: [StateDefToken,] },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @template S
 * @param {?} expression
 * @param {?=} context
 * @return {?}
 */
function ReduxSelect(expression, context) {
    return (target, propertyKey) => {
        let /** @type {?} */ observable;
        Object.defineProperty(target, propertyKey, {
            configurable: true,
            enumerable: true,
            get: () => {
                // use a getter to be lazy
                if (!observable) {
                    observable = new ReduxSelector(expression, context).distinctUntilChanged();
                }
                return observable;
            },
        });
    };
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @deprecated Use "ReduxSelector" instead
 */
class ReduxStateSelector {
    /**
     * @param {?} expression
     * @param {?=} context
     */
    constructor(expression, context) {
        this.selector = new ReduxSelector(expression, context);
    }
    /**
     * @return {?}
     */
    asObservable() {
        return this.selector;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
const ReduxState = ReduxStateDecorator.forClass;

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class ReduxStore extends InjectionToken {
    constructor() {
        super('ReduxStore');
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class ReduxCommonModule {
}
ReduxCommonModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    ReduxSelectPipe,
                ],
                exports: [
                    ReduxSelectPipe,
                ],
            },] },
];
/** @nocollapse */
ReduxCommonModule.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class ReduxTestingStore {
    constructor() {
        this.state = new BehaviorSubject$1(null);
    }
    /**
     * @template S
     * @param {?} state
     * @param {?} value
     * @return {?}
     */
    setState(state, value) {
        const { name } = ReduxStateDecorator.get(state);
        const /** @type {?} */ nextState = Object.assign({}, this.state.getValue(), {
            [name]: value,
        });
        this.state.next(nextState);
        return this.state
            .takeWhile(currentState => currentState !== nextState)
            .toPromise().then(() => this.state.getValue());
    }
    /**
     * @return {?}
     */
    getState() {
        return this.state.getValue();
    }
    /**
     * @param {?} listener
     * @return {?}
     */
    subscribe(listener) {
        return this.state.subscribe.call(this.state, listener);
    }
    /**
     * @return {?}
     */
    replaceReducer() {
    }
    /**
     * @template T
     * @param {?} action
     * @return {?}
     */
    dispatch(action) {
        return action;
    }
}
ReduxTestingStore.decorators = [
    { type: Injectable },
];
/** @nocollapse */
ReduxTestingStore.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class ReduxTestingModule {
    /**
     * @param {?=} store
     */
    constructor(store = null) {
        ReduxSelectorCacheFactory.clear();
        if (store) {
            Registry.reset();
            Registry.registerStore(store);
        }
    }
    /**
     * @param {?=} config
     * @return {?}
     */
    static forRoot(config = {}) {
        return {
            ngModule: ReduxTestingModule,
            providers: config.state ? [
                { provide: ReduxTestingStore, useFactory: config.storeFactory || ReduxTestingModule.defaultStoreFactory },
                { provide: StateDefToken, useValue: config.state || null, multi: true },
                config.state ? config.state.provider : null,
            ] : [
                { provide: ReduxTestingStore, useFactory: config.storeFactory || ReduxTestingModule.defaultStoreFactory },
                { provide: StateDefToken, useValue: config.state || null, multi: true },
            ],
        };
    }
    /**
     * @return {?}
     */
    static defaultStoreFactory() {
        return new ReduxTestingStore();
    }
}
ReduxTestingModule.decorators = [
    { type: NgModule, args: [{
                exports: [
                    ReduxCommonModule,
                ],
                imports: [
                    CommonModule,
                ],
                providers: [
                    ReduxTestingStore,
                ],
            },] },
];
/** @nocollapse */
ReduxTestingModule.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [ReduxTestingStore,] },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class StateDefinitionManager {
    /**
     * @param {?} injector
     */
    constructor(injector) {
        this.injector = injector;
        this.knownStateDefinitions = {};
    }
    /**
     * @param {?} stateDefs
     * @return {?}
     */
    add(stateDefs) {
        stateDefs
            .filter(def => def != null)
            .forEach(def => {
            const { name } = ReduxStateDecorator.get(def.provider);
            if (!this.knownStateDefinitions[name]) {
                Registry.registerState(this.injector.get(def.provider));
                if (Array.isArray(def.reducers)) {
                    this.registerReducers(name, def.reducers);
                }
                this.knownStateDefinitions[name] = def;
            }
        });
    }
    /**
     * @param {?} stateName
     * @param {?} reducers
     * @return {?}
     */
    registerReducers(stateName, reducers) {
        reducers.forEach(reducerClass => Object.values(reducerClass.prototype)
            .forEach(reducerMethod => {
            const /** @type {?} */ type = ReduxReducerDecorator.get(reducerMethod);
            Registry.registerReducer(stateName, Array.isArray(type) ? type : [type], reducerMethod);
        }));
    }
}
StateDefinitionManager.decorators = [
    { type: Injectable },
];
/** @nocollapse */
StateDefinitionManager.ctorParameters = () => [
    { type: Injector, },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class ReduxRootModule {
    /**
     * @param {?=} store
     * @param {?=} stateDefinitionManager
     * @param {?=} stateDefs
     */
    constructor(store = null, stateDefinitionManager, stateDefs = []) {
        this.stateDefinitionManager = stateDefinitionManager;
        Registry.registerStore(store);
        this.stateDefinitionManager.add(stateDefs);
    }
}
ReduxRootModule.decorators = [
    { type: NgModule, args: [{
                exports: [
                    ReduxCommonModule,
                ],
                imports: [
                    CommonModule,
                ],
                providers: [
                    StateDefinitionManager,
                ]
            },] },
];
/** @nocollapse */
ReduxRootModule.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [ReduxStore,] },] },
    { type: StateDefinitionManager, },
    { type: Array, decorators: [{ type: Optional }, { type: Inject, args: [StateDefToken,] },] },
];
class ReduxModule {
    /**
     * @param {?=} stateDefs
     * @param {?=} stateDefinitionManager
     */
    constructor(stateDefs = [], stateDefinitionManager) {
        this.stateDefinitionManager = stateDefinitionManager;
        this.stateDefinitionManager.add(stateDefs);
    }
    /**
     * @param {?=} config
     * @return {?}
     */
    static forChild(config = {}) {
        return {
            ngModule: ReduxModule,
            providers: [
                { provide: StateDefToken, useValue: config.state || null, multi: true },
                config.state ? config.state.provider : null,
            ],
        };
    }
    /**
     * @param {?=} config
     * @return {?}
     */
    static forRoot(config = {}) {
        return {
            ngModule: ReduxRootModule,
            providers: config.state ? [
                { provide: ReduxStore, useFactory: config.storeFactory || ReduxModule.defaultStoreFactory },
                { provide: StateDefToken, useValue: config.state || null, multi: true },
                config.state ? config.state.provider : null,
            ] : [
                { provide: ReduxStore, useFactory: config.storeFactory || ReduxModule.defaultStoreFactory },
                { provide: StateDefToken, useValue: config.state || null, multi: true },
            ],
        };
    }
    /**
     * @return {?}
     */
    static defaultStoreFactory() {
        return createStore(ReduxModuleRootReducer.reduce, {}, ReduxModule.defaultEnhancerFactory());
    }
    /**
     * @return {?}
     */
    static defaultEnhancerFactory() {
        if (console && window['__REDUX_DEVTOOLS_EXTENSION__'] && isDevMode()) {
            return window['__REDUX_DEVTOOLS_EXTENSION__']();
        }
        return ReduxModule.noopEnhancer;
    }
    /**
     * @param {?} next
     * @return {?}
     */
    static noopEnhancer(next) {
        return next;
    }
}
ReduxModule.decorators = [
    { type: NgModule, args: [{
                exports: [
                    ReduxCommonModule,
                ],
                imports: [
                    CommonModule,
                ],
                providers: [
                    StateDefinitionManager,
                ]
            },] },
];
/** @nocollapse */
ReduxModule.ctorParameters = () => [
    { type: Array, decorators: [{ type: Optional }, { type: Inject, args: [StateDefToken,] },] },
    { type: StateDefinitionManager, },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Generated bundle index. Do not edit.
 */

export { ReduxAction, ReduxActionContext, ActionInterface, ReduxModuleRootReducer, ReduxReducer, ReduxSelect, ReduxSelectPipe, ReduxState, ReduxStateSelector, ReduxStore, ReduxTestingModule, ReduxTestingStore, ReduxSelector, ReduxSelectorCacheFactory, ReduxRootModule, ReduxModule, ReduxCommonModule as ɵb, StateDefinitionManager as ɵc, StateDefToken as ɵa };
export { ReduxActionDecorator, ReduxActionDispatcher, ReduxActionContextDecorator, ReduxModuleDecorator, ReduxReducerDecorator, ReduxStateDecorator } from '@harmowatch/redux-decorators';
//# sourceMappingURL=harmowatch-ngx-redux-core.js.map
